#include <Arduino.h>
#include <Wire.h>

#define PORT_A_BASE 0x40010800
#define PORT_B_BASE 0x40010c00

#define CRL_OFFSET 0x00
#define CRH_OFFSET 0x04
#define IDR_OFFSET 0x08
#define ODR_OFFSET 0x0c

/*#define AFIO_BASE 0x40010000
#define MAPR_OFFSET 0x04*/

/*#define RESET_BASE 0x40021000
#define APB_OFFSET 0x10
#define I2C1_OFFSET 0x200000*/


#define H_INPUT_MASK 0x0000ffff

//nastaveni pinu pro vstup s rezistorem v port configuration register (na kazdy pin 4 bity)
#define INPUT_PULLUP_CONF 0b1000


#define ADDRESS 0x42

#define AUX_PORT_WIDTH 8
#define AUX_PORT_AMOUNT 2

#define TOKEN_PIN_IN PA8
#define TOKEN_PIN_OUT PA9

#define END_CHAR 0xff
#define EMPTY_CHAR (uint8_t)0xff

#define AFIO_MAPR_OFFSET 0x04

#define I2C_CONTROL1 0x00
#define I2C_CONTROL2 0x04

#define TIMEOUT 100

#define NO_INDEX 255

int pin_seq[] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7,
					PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15};

int reads = 0;
int row = 0;

unsigned char device_index = NO_INDEX;
char init_done = 0;
char aux_devices = 0;

char active_pin_index = 0;


int CR2_REG = 0x324;

void pin_setup() {
	//naplneni reg_value hodnotou pro vstup s rezistorem pro vsech 8 pinu ktere registr ovlada
	/*int reg_value = INPUT_PULLUP_CONF << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF) << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF) << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF) << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF) << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF) << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF) << 4;
	reg_value = (reg_value | INPUT_PULLUP_CONF);
	
	//na portu A chci jako vstupni piny 0 - 7 (tj. CRL)
	*(int *)(PORT_A_BASE + CRL_OFFSET) = reg_value;
	//na portu B chci jako vstupni piny 8 - 15 (tj. CRH)
	*(int *)(PORT_B_BASE + CRH_OFFSET) = reg_value;

	//nastaveni PULLUP rezistoru zapisem do ODR registru, port A piny 0-7
	*(int *)(PORT_A_BASE + ODR_OFFSET) = 0x000000ff;
	//nastaveni PULLUP rezistoru zapisem do ODR registru, port B piny 8-15
	*(int *)(PORT_B_BASE + ODR_OFFSET) = 0x0000ff00;*/

	/*LL_GPIO_SetPinMode(GPIOA, LL_GPIO_PIN_0 | LL_GPIO_PIN_3, LL_GPIO_MODE_INPUT);
	LL_GPIO_SetPinPull(GPIOA, LL_GPIO_PIN_0 | LL_GPIO_PIN_3, LL_GPIO_PULL_UP);*/

	for(int i = 0; i < AUX_PORT_AMOUNT * AUX_PORT_WIDTH; ++i) {
		pinMode(pin_seq[i], INPUT_PULLUP);
	}
}

void shift_pin() {
	//predchozi aktivni pin vratim do neutralni urovne
	if(active_pin_index == 0) {
		pinMode(pin_seq[AUX_PORT_AMOUNT * AUX_PORT_WIDTH - 1], INPUT_PULLUP);
	}
	else {
		pinMode(pin_seq[(active_pin_index - 1)], INPUT_PULLUP);
	}
	
	//aktivuju dalsi pin, pokud je na moji desce
	if(row >= AUX_PORT_AMOUNT * AUX_PORT_WIDTH * device_index && row < AUX_PORT_AMOUNT * AUX_PORT_WIDTH * (device_index + 1)) {
		pinMode(pin_seq[active_pin_index], OUTPUT);
		digitalWrite(pin_seq[active_pin_index], LOW);
		active_pin_index = (active_pin_index + 1) % (AUX_PORT_AMOUNT * AUX_PORT_WIDTH);
	}
}

void shift_pin_req(int byte_nr) {
	unsigned long time = 0;
	//digitalWrite(PC13, HIGH);

	if(byte_nr == 0) {
		return;
	}

	time = millis();
	//explicitni timeout, kdyby doslo k nejake vazne chybe
	while(!Wire.available()) {
		if(millis() - time > TIMEOUT) {
			return;
		}
	}
	//data se muzou zahodit
	Wire.read();

	//posunu radek, pozadavek na posun by nemel prijit v prubehu jedne radky
	row = (row + 1) % (AUX_PORT_AMOUNT * AUX_PORT_WIDTH);
	shift_pin();
}

void state_send() {
	uint8_t data = 0;
	int port = 0;
	digitalWrite(PC13, LOW);

	//if(reads >= (AUX_PORT_AMOUNT * AUX_PORT_WIDTH * device_index) / 8 && reads < (AUX_PORT_AMOUNT * AUX_PORT_WIDTH * (device_index + 1)) / 8) {
	if(reads == (AUX_PORT_AMOUNT * AUX_PORT_WIDTH * device_index) / 8) {
		//z portu A me zajimaji bity 7-0, vzhledem k pullupum budou aktivni piny nizke, to si zneguje ESP
		port = LL_GPIO_ReadInputPort(GPIOA);
		data = ((uint8_t *)&port)[0];
		Wire.write(data);
		//Wire.write((uint8_t)(*(uint8_t *)(PORT_A_BASE + IDR_OFFSET) + 3));
		//z portu B me zajimaji bity 15-8
		//Wire.write((uint8_t)(*(uint8_t *)(PORT_B_BASE + IDR_OFFSET) + 2));

		digitalWrite(TOKEN_PIN_OUT, HIGH);
	}
	else if(reads = (AUX_PORT_AMOUNT * AUX_PORT_WIDTH * device_index) / 8 + 1) {
		//z portu B me zajimaji bity 8-15
		port = LL_GPIO_ReadInputPort(GPIOB);
		data = ((uint8_t *)&port)[1];
		Wire.write(data);
	}
	else { //nemuzu vysilat
		Wire.write(EMPTY_CHAR);
		//pokud jsem predaval povereni dale, muzu ho vypnout
		if(digitalRead(TOKEN_PIN_OUT)) {
			digitalWrite(TOKEN_PIN_OUT, LOW);
		}
	}

	++reads;
	if(reads == (AUX_PORT_AMOUNT * AUX_PORT_WIDTH * aux_devices) / 8) {
		row = (row + 1) % (AUX_PORT_AMOUNT * AUX_PORT_WIDTH * aux_devices);
		//staci cist az od registru, ve kterem je aktivni pin
		reads = row / 8;
		shift_pin();
	}
}

void init_send() {
	digitalWrite(PC13, HIGH);
	uint8_t to_send = ADDRESS;
	//digitalWrite(PC15, HIGH);

	//pokud nemam token, "neodpovidam"
	if(digitalRead(TOKEN_PIN_IN) == LOW) {
		Wire.write(EMPTY_CHAR);
		return;
	}
	if(device_index == NO_INDEX) {
		//pokud jsem jeste neobdrzel index, tak odpovidam nejakou libolnou nekoncovou zpravou
		Wire.write(to_send);
		return;
	}
	Wire.write(EMPTY_CHAR);
	return;
}

void init_recv(int byte_nr) {
	unsigned long time = 0;
	uint8_t received = 0;
	//kazda zprava z ESP znamena jeden mikrokontroler
	digitalWrite(PC13, HIGH);
	++aux_devices;

	if(byte_nr == 0) {
		return;
	}

	time = millis();
	//explicitni timeout, kdyby doslo k nejake vazne chybe
	while(!Wire.available()) {
		if(millis() - time > TIMEOUT) {
			return;
		}
	}
	received = Wire.read();

	if(digitalRead(TOKEN_PIN_IN) == LOW) {
		//data muzu rovnou zahodit pokud nemam token
		return;
	}
	if(device_index == NO_INDEX) {
		//mam token ale jeste ne index - to co jsem prijal je muj index
		device_index = received;
		digitalWrite(TOKEN_PIN_OUT, HIGH);

		digitalWrite(PC13, HIGH);
		return;
	}
	if(received == END_CHAR) {
		//pokud je znak koncovy, tak tim konci inicializace
		init_done = 1;
		//posledni zprava dalsi kontroler neznamenala
		--aux_devices;
		Wire.onRequest(state_send);
		Wire.onReceive(shift_pin_req);
		shift_pin();
		return;
	}
	//pokud ne, data zahazuju a pokracuje se
}

/*void change_i2c_state() {
	*(int *)(AFIO_BASE + AFIO_MAPR_OFFSET) = *(int *)(AFIO_BASE + AFIO_MAPR_OFFSET) & ~AFIO_MAPR_I2C1_REMAP;
	digitalWrite(PC13, LOW);
}*/

/*void change_i2c_state() {
	//zmenim nejnizsi bit control registru = peripheral enable/disable
	*(int *)(I2C1_BASE + I2C_CONTROL2) = CR2_REG;
	*(int *)(I2C1_BASE + I2C_CONTROL1) = *(int *)(I2C1_BASE + I2C_CONTROL1) | 0x01;
	digitalWrite(PC13, LOW);
}*/


void setup() {
	pin_setup();
	pinMode(PC13, OUTPUT);
	digitalWrite(PC13, LOW);

	pinMode(PC15, OUTPUT);
	/*Wire.setSCL(PB6);
	Wire.setSDA(PB7);*/

  	//I2C je na PB6 (SCL) a PB7 (SDA)
	Wire.begin(ADDRESS, 400000);
	
	Wire.onReceive(init_recv);
	Wire.onRequest(init_send);

	//*(int *)(I2C1_BASE + I2C_CONTROL1) = *(int *)(I2C1_BASE + I2C_CONTROL1) & 0xfffffffe;
	
	//Serial.begin(9600);
  	
	//attachInterrupt(digitalPinToInterrupt(TOKEN_PIN_IN), change_i2c_state, RISING);
	//attachInterrupt(digitalPinToInterrupt(TOKEN_PIN_IN), disable_i2c, LOW);

	pinMode(TOKEN_PIN_IN, INPUT_PULLDOWN);
	pinMode(TOKEN_PIN_OUT, OUTPUT);
	digitalWrite(TOKEN_PIN_OUT, LOW);

}

void loop() {

	/*int read = *(int *)(PORT_A_BASE + IDR_OFFSET);
	read = read & 0xff;
	Serial.printf("Port A: %x\n", read);

	read = *(int *)(PORT_B_BASE + IDR_OFFSET);
	read = read & 0xff00;
	Serial.printf("Port B: %x\n", read);*/

	
}
