#include "logic.h"

extern block blocks[BLOCK_AMOUNT];
extern pin pins[PIN_AMOUNT];
extern wire wires[WIRE_AMOUNT];

extern unsigned long sim_time;
extern char sim_time_enable;

unsigned long but_debounce[] = {0, 0};
state but_last_state[] = {STATE_0, STATE_0};

wire_number wire_from_pin(pin_number p) {
	if(p <= PIN_AMOUNT) {
		return *pins[p].wire_nr;
	}
	return 0;
}

char execute_block(block_number b) {
	unsigned char input_number = 0;
	unsigned char result = 0;
	state input_states[INPUT_NR];
	state output_states[OUTPUT_NR];
	pin_number *input_pins = blocks[b].input;
	wire_number input_wire = 0;

	for(input_number; input_number < blocks[b].inputs_used; ++input_number) {
		input_wire = wire_from_pin(input_pins[input_number]);
		/*if(!input_wire) {
			break;
		}*/

		input_states[input_number] = wires[input_wire].wire_state;
	}

	//nez funkci zavolam, overim si, ze vubec je co zavolat
	if(blocks[b].function_pointer) {
		result = blocks[b].function_pointer(input_states, input_number, output_states);
	}

	//nektere funkcni bloky muzou vratit NG, pokud si nepreji menit predchozi vystup
	if(result == NG) {
		return 0;
	}
	
	//pokud se bere v uvahu zpozdeni, musi byt vystupy nejakou dobu stabilni, nez se propaguji "ven"
	if(sim_time_enable == TIME_ON) {
		//podivam se, jestli se nove vystupy lisi od tech predchozich, pokud ano, doba propagace se oddali
		for(int i = 0; i < OUTPUT_NR; ++i) {
			if(!blocks[b].output[i]) {
				break;
			}
			if(output_states[i] != blocks[b].new_pin_states[i]) {
				blocks[b].next_update = sim_time + BLOCK_DELAY;
				blocks[b].new_pin_states[i] = output_states[i];
			}
		}
	}

	//pokud jeste nenastal cas aktualizace, tak tim funkce konci
	if(blocks[b].next_update > sim_time) {
		return 0;
	}

	//pokud uz cas nastal, vystupy se skutecne zapisou
	for(int i = 0; i < OUTPUT_NR; ++i) {
		if(!blocks[b].output[i]) {
			break;
		}

		//zadny vystup nema byt nedefinovany
		//pokud nedefinovany je, tak nekde doslo k chybe (napr nezapojeny vstup)
		if(output_states[i] == STATE_UNDEF) {
			show_error(RUN_ERR, 0, blocks[b].output[i]);
			return 0;
		}
		write_outputs(output_states[i], blocks[b].output[i]);
	}

	return 0;
}

/**
 * @brief Zapise na drat specifikovany argumentem output hodnotu result
 * Zapisuje pouze v pripade, ze jsou vsechny ostatni vystupy na drate ve stavu high-Z, jinak vraci chybu
 * 
 * @param result stav, jaky se ma na drat zapsat
 * @param from z jakeho pinu se stav na drat zapisuje
 */
char write_outputs(state result, pin_number from) {
	wire_number dest_wire = wire_from_pin(from);
	pin_number drive_pin = 0;

	#ifdef PRINT
	Serial.printf("Zapis na drat: %d\n", (int)dest_wire);
	Serial.printf("Zdroj pin: %d\n", (int)from);
	#endif

	//stav zapisu na konkretni pin
	pins[from].pin_state = result;

	if(!dest_wire) {
		//pokud se vyskytne soucastka, ktera neni nikam zapojena (jeji pin bude na drate 0), preskakuji
		return 0;
	}

	for(int i = 0; i < OUTPUT_NR; ++i) {
		if(!wires[dest_wire].outputs[i]) {
			break; //nula v poli znamena konec
		}

		if(wires[dest_wire].outputs[i] == from) {
			continue; //"zdrojovy" pin neresim
		}

		//musim zkontrolovat, ze vsechny pripadne ostatni vystupy jsou v HIGHZ
		if(pins[wires[dest_wire].outputs[i]].pin_state != STATE_HIGHZ) {
			//nejaky pin neni ve stavu vysoke impedance
			//je muj pin ve stavu vysoke impedance? pokud ano, je to v poradku
			if(result != STATE_HIGHZ) {
				#ifdef PRINT
				Serial.printf("Spojeny dva vodice natvrdo!\n");
				#endif
				
				show_error(RUN_ERR, 0, wires[dest_wire].outputs[i]);
				//runtime_error(wires[dest_wire].outputs[i]);
				wires[dest_wire].wire_state = STATE_UNDEF;
				return 1;
			}
			
			//pokud by zadny pin neudaval hodnotu vodice, zapise se HIGHZ, jinak zustane predchozi stav vodice
			drive_pin = wires[dest_wire].outputs[i];
		}
	}
	
	if(!drive_pin) {
		wires[dest_wire].wire_state = result;
	}
	else {
		//pokud nedoslo ke stretu vystupu, muzu ten muj zapsat
		wires[dest_wire].wire_state = pins[drive_pin].pin_state;
	}

	return 0;
}

char and_func(state *inputs, unsigned char input_number, state *outputs) {
	//AND ma pouze jediny vystup
	unsigned char output_index = 0;

	outputs[output_index] = STATE_1;

	#ifdef PRINT
	Serial.println("Zpracovavam AND.");
	#endif

	//vypoctu vystup fcniho bloku
	for(int i = 0; i < input_number; ++i) {
		if(inputs[i] == STATE_0) { //pokud je stav alespon jednoho dratu nizky, vystup bloku bude take nizky nezavisle na ostatnich
			outputs[output_index] = STATE_0;

			#ifdef PRINT
			Serial.println("Jeden vstup nizky, vystup nizky.");
			#endif

			break;
		}

		//pokud by zadny vstup nebyl nizky, musim vedet, jestli nejaky nebyl nedefinovany nebo v highZ (staci zjistovat MSB)
		//pokud ano, vystup bude nedefinovany
		if(inputs[i] & STATE_UNDEF) {
			outputs[output_index] = STATE_UNDEF;
		}
	}

	#ifdef PRINT
	Serial.printf("Hodnota vysledku: %d\n", outputs[output_index]);
	#endif

	return OK;
}

/*void and_func(pin_number *inputs, pin_number *outputs) {
	state result = STATE_1;

	#ifdef PRINT
	Serial.println("Zpracovavam AND.");
	#endif

	//vypoctu vystup fcniho bloku
	for(int i = 0; i < INPUT_NR; ++i) {
		if(!inputs[i]) { //ukazatel na nulove cislo dratu znamena konec obsazene casti pole
			break;
		}
		if(wires[*pins[inputs[i]].wire_nr].wire_state == STATE_0) { //pokud je stav alespon jednoho dratu nizky, vystup bloku bude take nizky nezavisle na ostatnich
			result = 0;

			#ifdef PRINT
			Serial.println("Jeden vstup nizky, vystup nizky.");
			#endif

			break;
		}

		//pokud by zadny vstup nebyl nizky, musim vedet, jestli nejaky nebyl nedefinovany nebo v highZ (staci zjistovat MSB)
		//pokud ano, vystup bude nedefinovany
		if(wires[*pins[inputs[i]].wire_nr].wire_state & STATE_UNDEF) {
			result = STATE_UNDEF;
		}

		#ifdef PRINT
		Serial.printf("Hodnota vysledku: %d\n", result);
		#endif
	}

	//and ma pouze jeden vystup, na ten se zapise vysledek
	write_outputs(result, outputs[0]);
}*/

char nand_func(state *inputs, unsigned char input_number, state *outputs) {
	//NAND ma pouze jediny vystup
	unsigned char output_index = 0;

	outputs[output_index] = STATE_0;

	#ifdef PRINT
	Serial.println("Zpracovavam NAND.");
	#endif

	//vypoctu vystup fcniho bloku
	for(int i = 0; i < input_number; ++i) {
		if(inputs[i] == STATE_0) { //pokud je stav alespon jednoho vstupu nizky, vystup bloku bude vysoky nezavisle na ostatnich
			outputs[output_index] = STATE_1;

			#ifdef PRINT
			Serial.println("Jeden vstup nizky, vystup vysoky.");
			#endif

			break;
		}
		//pokud by zadny vstup nebyl nizky, musim vedet, jestli nejaky nebyl nedefinovany nebo v highZ, tzn. jestli nemel MSB == 1
		//pokud ano, vystup bude nedefinovany
		if(inputs[i] & STATE_UNDEF) {
			outputs[output_index] = STATE_UNDEF;
		}
		//result = result | (wires[*pins[inputs[0]].wire_nr].wire_state & STATE_UNDEF);
	}

	#ifdef PRINT
	Serial.printf("Hodnota vysledku: %d\n", outputs[output_index]);
	#endif

	return OK;
}

char buffer(state *inputs, unsigned char input_number, state *outputs) {
	//buffer ma pouze jediny vystup
	unsigned char output_index = 0;
	
	#ifdef PRINT
	Serial.println("Zpracovavam budic.");
	#endif
	outputs[output_index] = STATE_UNDEF;
	
	//pokud je druhy vstup aktivni, prvni se dostane na vystup
	if(inputs[1] == STATE_1) {
		outputs[output_index] = inputs[0];
	}
	//pokud je ridici vstup nizky, vystup je HIGHZ
	else if(inputs[1] == STATE_0) {
		outputs[output_index] = STATE_HIGHZ;
	}
	//pokud je na ridicm vstupu cokoliv jineho nez vysoky ci nizky, vystup je nedefinovany (na ten probehla inicializace vystupu)
	return OK;
}

/*char and3(pin_number *inputs, pin_number *outputs) {
  
	//vypoctu vystup fcniho bloku
	//logicky operator vrati 1 (nebo 0), posun na MSB pro jednodussi shiftovani dale
	unsigned char result = ((i & 0b10000000) && (i & 0b01000000) && (i & 0b00100000)) << 7;
	write_outputs(result, o);
}*/

char or_func(state *inputs, unsigned char input_number, state *outputs) {
	//jediny vystup
	unsigned char output_index = 0;

	outputs[output_index] = STATE_0;

	#ifdef PRINT
	Serial.println("Zpracovavam OR.");
	#endif

	//vypoctu vystup fcniho bloku
	for(int i = 0; i < input_number; ++i) {
		if(inputs[i] == STATE_1) { //pokud je stav alespon jednoho dratu vysoky, vystup bloku bude take vysoky nezavisle na ostatnich
			outputs[output_index] = STATE_1;

			#ifdef PRINT
			Serial.println("Jeden vstup vysoky, vystup vysoky.\n");
			#endif

			break;
		}
		//pokud by zadny vstup nebyl vysoky, musim vedet, jestli nejaky nebyl nedefinovany nebo v highZ
		//pokud ano, vystup bude nedefinovany
		if(inputs[i] & STATE_UNDEF) {
			outputs[output_index] = STATE_UNDEF;
		}

		#ifdef PRINT
		Serial.printf("Hodnota vysledku: %d\n", outputs[output_index]);
		#endif
	}

	return OK;
}

char nor_func(state *inputs, unsigned char input_number, state *outputs) {
	//jediny vystup
	unsigned char output_index = 0;

	outputs[output_index] = STATE_1;

	#ifdef PRINT
	Serial.println("Zpracovavam NOR.");
	#endif

	//vypoctu vystup fcniho bloku
	for(int i = 0; i < input_number; ++i) {
		if(inputs[i] == STATE_1) { //pokud je stav alespon jednoho dratu vysoky, vystup bloku bude nizky nezavisle na ostatnich
			outputs[output_index] = STATE_0;

			#ifdef PRINT
			Serial.println("Jeden vstup vysoky, vystup nizky.\n");
			#endif

			break;
		}
		//pokud by zadny vstup nebyl vysoky, musim vedet, jestli nejaky nebyl nedefinovany nebo v highZ
		//pokud ano, vystup bude nedefinovany
		if(inputs[i] & STATE_UNDEF) {
			outputs[output_index] = STATE_UNDEF;
		}

		#ifdef PRINT
		Serial.printf("Hodnota vysledku: %d\n", outputs[output_index]);
		#endif
	}

	return OK;
}


char not_func(state *inputs, unsigned char input_number, state *outputs) {
	//jediny vystup
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	Serial.println("Zpracovavam NOT.");
	#endif
	
	if(inputs[0] == STATE_1) {
		outputs[output_index] = STATE_0;
	}
	else if(inputs[0] == STATE_0) {
		outputs[output_index] = STATE_1;
	}
	
	//pokud na vstupu nebyla 1 ani 0, vystup je UNDEF

	#ifdef PRINT
	Serial.printf("Hodnota vysledku: %d\n", outputs[output_index]);
	#endif

	return OK;
}

char led0(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	Serial.printf("LED\n");
	Serial.printf("Vstupni hodnota: %d\n", (int)inputs[0]);
	#endif
	
	//pokud je vstup vysoky, rozsviti LED
	//vystup kopiruje vstup
	if(inputs[0] == STATE_1) {
    	digitalWrite(LED0_PIN, HIGH);
    	outputs[output_index] = STATE_1;
	}
	else if(inputs[0] == STATE_0) {
		digitalWrite(LED0_PIN, LOW);
    	outputs[output_index] = STATE_0;
	}
	//pokud vstup neni ani 1 ani 0, je vystup nedefinovany a stav LED se nemeni
	#ifdef PRINT
	Serial.printf("Vystupni hodnota: %d\n", outputs[output_index]);
	#endif
	return OK;
}

char led1(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	Serial.printf("LED\n");
	Serial.printf("Vstupni hodnota: %d\n", (int)inputs[0]);
	#endif
	
	//pokud je vstup vysoky, rozsviti LED
	//vystup kopiruje vstup
	if(inputs[0] == STATE_1) {
    	digitalWrite(LED1_PIN, HIGH);
    	outputs[output_index] = STATE_1;
	}
	else if(inputs[0] == STATE_0) {
		digitalWrite(LED1_PIN, LOW);
    	outputs[output_index] = STATE_0;
	}
	//pokud vstup neni ani 1 ani 0, je vystup nedefinovany a stav LED se nemeni
	#ifdef PRINT
	Serial.printf("Vystupni hodnota: %d\n", outputs[output_index]);
	#endif
	return OK;
}

char led2(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	Serial.printf("LED\n");
	Serial.printf("Vstupni hodnota: %d\n", (int)inputs[0]);
	#endif
	
	//pokud je vstup vysoky, rozsviti LED
	//vystup kopiruje vstup
	if(inputs[0] == STATE_1) {
    	digitalWrite(LED2_PIN, HIGH);
    	outputs[output_index] = STATE_1;
	}
	else if(inputs[0] == STATE_0) {
		digitalWrite(LED2_PIN, LOW);
    	outputs[output_index] = STATE_0;
	}
	//pokud vstup neni ani 1 ani 0, je vystup nedefinovany a stav LED se nemeni
	#ifdef PRINT
	Serial.printf("Vystupni hodnota: %d\n", outputs[output_index]);
	#endif
	return OK;
}

char led3(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	Serial.printf("LED\n");
	Serial.printf("Vstupni hodnota: %d\n", (int)inputs[0]);
	#endif
	
	//pokud je vstup vysoky, rozsviti LED
	//vystup kopiruje vstup
	if(inputs[0] == STATE_1) {
    	digitalWrite(LED3_PIN, HIGH);
    	outputs[output_index] = STATE_1;
	}
	else if(inputs[0] == STATE_0) {
		digitalWrite(LED3_PIN, LOW);
    	outputs[output_index] = STATE_0;
	}
	//pokud vstup neni ani 1 ani 0, je vystup nedefinovany a stav LED se nemeni
	#ifdef PRINT
	Serial.printf("Vystupni hodnota: %d\n", outputs[output_index]);
	#endif
	return OK;
}

/*char led2(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	Serial.printf("LED");
	Serial.printf("Chci cist z pinu: %d\n", (int)inputs[0]);
	#endif
	
	//pokud je vstup vysoky, rozsviti LED
	//vystup kopiruje vstup
	if(inputs[0] == STATE_1) {
    	digitalWrite(LED0_PIN, HIGH);
    	outputs[output_index] = STATE_1;
	}
	else if(inputs[0] == STATE_0) {
		digitalWrite(LED0_PIN, LOW);
    	outputs[output_index] = STATE_0;
	}
	//pokud vstup neni ani 1 ani 0, je vystup nedefinovany a stav LED se nemeni
	return OK;
}*/

char motor(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	outputs[output_index] = STATE_UNDEF;

	#ifdef PRINT
	printf("Motor");
	printf("Chci cist z pinu: %d\n", (int)inputs[0]);
	#endif
	
	//pokud je vstup vysoky, motor se roztoci
	if(inputs[0] == STATE_1) {
		ledcWrite(MOTOR_PWM_CHANNEL, MOTOR_ON);
    	outputs[output_index] = STATE_1;
	}
	else if(inputs[0] == STATE_0) {
		ledcWrite(MOTOR_PWM_CHANNEL, MOTOR_OFF);
    	outputs[output_index] = STATE_0;
	}

	return OK;
}

char fotoodpor0(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	int input = analogRead(FOTOR0_PIN);

	#ifdef PRINT
	Serial.printf("Zpracovavam fotorezistor.\n");
	Serial.printf("Nacteno z pinu: %d\n", input);
	#endif

	//inicializace na nizkou uroven
	outputs[output_index] = STATE_0;
	//pokud je dost svetla, tak se prepise na 1
	if(input < FOTOR_THRESHOLD) {
		outputs[output_index] = STATE_1;
	}

	#ifdef PRINT
	Serial.printf("Nactena hodnota: %d\n", outputs[output_index]);
	#endif

	return OK;
}

char fotoodpor1(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	int input = analogRead(FOTOR1_PIN);

	//inicializace na nizkou uroven
	outputs[output_index] = STATE_0;
	//pokud je dost svetla, tak se prepise na 1
	if(input < FOTOR_THRESHOLD) {
		outputs[output_index] = STATE_1;
	}

	return OK;
}

char low(state *inputs, unsigned char input_number, state *outputs) {
	//zdroj nizke logicke urovne
	outputs[0] = STATE_0;

	return OK;
}

char high(state *inputs, unsigned char input_number, state *outputs) {
	//zdroj vysoke logicke urovne
	outputs[0] = STATE_1;

	return OK;
}

char but0(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	//pokud neuplynul debounce, vystup zustava nemenny
	if(millis() - but_debounce[0] < DEBOUNCE_TIME) {
		//outputs[output_index] = but_last_state[0];
		return NG;
	}

	outputs[output_index] = STATE_0;
	
	if(digitalRead(BUT0_PIN)) {
		outputs[output_index] = STATE_1;
	}
  	#ifdef PRINT
	Serial.printf("Nacteno z tlacitka: %d\n", outputs[output_index]);
  	#endif

	return OK;
}

char but1(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	//pokud neuplynul debounce, vystup zustava nemenny
	if(millis() - but_debounce[1] < DEBOUNCE_TIME) {
		//outputs[output_index] = but_last_state[1];
		return NG;
	}
	
	outputs[output_index] = STATE_0;
	
	if(digitalRead(BUT1_PIN)) {
		outputs[output_index] = STATE_1;
	}
  	#ifdef PRINT
	Serial.printf("Nacteno z tlacitka: %d\n", outputs[output_index]);
  	#endif

	return OK;
}

char ir(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;
	
	//neni potreba resit debounce
	outputs[output_index] = STATE_0;
	
	if(digitalRead(IR_PIN)) {
		outputs[output_index] = STATE_1;
	}
  	#ifdef PRINT
	Serial.printf("Nacteno z tlacitka: %d\n", outputs[output_index]);
  	#endif

	return OK;
}

char digi_input(state *inputs, unsigned char input_number, state *outputs) {
	//prvni polozka pole inputs uchovava cislo GPIO, ze ktereho se ma stav tlacitka cist
	state result = STATE_0;
	
	if(digitalRead(inputs[0])) {
		result = STATE_1;
	}
  	#ifdef PRINT
	Serial.printf("Nacteno z tlacitka: %d\n", (int)result);
  	#endif

	write_outputs(result, outputs[0]);
	return OK;
}


char D_flipflop(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	#ifdef PRINT
	Serial.println("Zpracovavam D klopny obvod.");
	#endif

	//pokud je druhy vstup (CLK) aktivni, prvni se dostane na vystup
	if(inputs[1] == STATE_1) {
		outputs[output_index] = inputs[0];
		return OK;
	}
	//pokud CLK aktivni neni, nic se nedeje
	return NG;
}


char repro(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;
	
	#ifdef PRINT
	Serial.println("Zpracovavam reproduktor.");
	#endif

	//pokud je prvni vstup aktivni, repro zacne vydavat zvuk
	if(inputs[0] == STATE_1) {
		ledcWriteNote(REPRO_PWM_CHANNEL, NOTE_C, 2);
		outputs[output_index] = STATE_1;
	}
	else {
		ledcWrite(REPRO_PWM_CHANNEL, 0);
		outputs[output_index] = STATE_0;
	}
	return OK;
}


char servo(state *inputs, unsigned char input_number, state *outputs) {
	unsigned char output_index = 0;

	#ifdef PRINT
	Serial.println("Zpracovavam servo.");
	#endif

	//pokud je vstup aktivni, servo se otoci do jedne krajni polohy
	if(inputs[0] == STATE_1) {
		ledcWrite(SERVO_PWM_CHANNEL, SERVO_RIGHT);
		outputs[output_index] = STATE_1;
	}
	//pokud neni, otoci se do druhe krajni polohy
	else {
		ledcWrite(SERVO_PWM_CHANNEL, SERVO_LEFT);
		outputs[output_index] = inputs[0];
	}

	#ifdef PRINT
	Serial.printf("Vystup: %d\n", outputs[output_index]);
	#endif
	return OK;
}
