#ifndef logic
#define logic

//#define PRINT

#include <Arduino.h>
#include "main.h"

#define BLOCK_AMOUNT 100
#define PIN_AMOUNT 254
#define WIRE_AMOUNT 200

#define BLOCK_DELAY 10

#define OUTPUT_NR 10
#define INPUT_NR 10

#define STATE_UNDEF		0b10000000
#define STATE_HIGHZ 	0b10000001
#define STATE_1 		0b00100000
#define STATE_0 		0b00000000
#define STATE_THRESHOLD	0b00010000

#define IO_OUTPUT 0b10000000

#define TRISTATE_TRUE 1
#define TRISTATE_FALSE 0

#define BUT0_PIN 14
#define BUT1_PIN 15
#define DEBOUNCE_TIME 100

#define FOTOR0_PIN 34
#define FOTOR1_PIN 39
#define FOTOR_THRESHOLD 2000

#define IR_PIN 35

#define LED0_PIN 5
#define LED1_PIN 13
#define LED2_PIN 22
#define LED3_PIN 19

#define TOKEN_PIN 2

#define SDA_PIN 18
#define SCL_PIN 23

#define TM1638_STROBE 32
#define TM1638_CLOCK 33
#define TM1638_DATA 25

#define REPRO_PIN 16
#define REPRO_PWM_CHANNEL 1

#define SERVO_PIN 17
#define SERVO_PWM_CHANNEL 3
#define SERVO_FREQ 50
#define SERVO_BIT_DEPTH 10
//strida pro krajni polohy 50Hz serva pri 10 bitovem rozliseni (0 - 1024)
#define SERVO_LEFT 30
#define SERVO_RIGHT 120

#define MOTOR_PIN 12
#define MOTOR_PWM_CHANNEL 2
#define MOTOR_PWM_FREQ 50
#define MOTOR_BIT_DEPTH 10
#define MOTOR_ON 500
#define MOTOR_OFF 0

#define TIME_OFF 0
#define TIME_ON 1

#define OK 0
#define NG 1

typedef unsigned char pin_number;

typedef unsigned char block_number;

typedef unsigned char wire_number;

typedef unsigned char state;

typedef char (*fp)(state *inputs, unsigned char input_number, state *outputs);


typedef struct {
	//odkaz na funkci, kterou blok vykonava
	fp function_pointer;

	//cisla vstupnich pinu bloku
	pin_number input[INPUT_NR];

	//pocet skutecne vyuzitych vstupu
	unsigned char inputs_used;

	//cisla vystupnich pinu bloku
	pin_number output[OUTPUT_NR];

	//budouci hodnoty bloku
	state new_pin_states[OUTPUT_NR];

	//cas, kdy bude mozne aktualizovat vystup
	unsigned long next_update;
} block;

typedef struct {
  	//cislo vstupu nebo vystupu bloku, kteremu dany pin odpovida
  	unsigned char io_nr;

  	//cislo bloku, kteremu dany pin odpovida
  	block_number block_nr;

  	//stav pinu
  	state pin_state;

  	//cislo dratu, na ktery je pin pripojen
  	wire_number *wire_nr;

  	//priznak, jestli pin umi tri stavy (teoreticky by to slo zakomponovat do io_nr, ale pro prehlednost samostatne)
  	unsigned char tristate;
} pin;

typedef struct {
  //cisla pinu, ktere cerpaji hodnotu z dratu
  pin_number *inputs;

  //cisla pinu, ktere zapisuji hodnotu na drat
  pin_number *outputs;

  //stav dratu
  state wire_state;
} wire;

char execute_block(block_number b);

char write_outputs(unsigned char result, unsigned char from);

//void and_func(pin_number *inputs, pin_number *outputs);
char and_func(state *inputs, unsigned char input_number, state *outputs);

char nand_func(state *inputs, unsigned char input_number, state *outputs);

char or_func(state *inputs, unsigned char input_number, state *outputs);

char nor_func(state *inputs, unsigned char input_number, state *outputs);

char not_func(state *inputs, unsigned char input_number, state *outputs);

char led0(state *inputs, unsigned char input_number, state *outputs);
char led1(state *inputs, unsigned char input_number, state *outputs);
char led2(state *inputs, unsigned char input_number, state *outputs);
char led3(state *inputs, unsigned char input_number, state *outputs);

char fotoodpor0(state *inputs, unsigned char input_number, state *outputs);

char fotoodpor1(state *inputs, unsigned char input_number, state *outputs);

char buffer(state *inputs, unsigned char input_number, state *outputs);

char low(state *inputs, unsigned char input_number, state *outputs);

char high(state *inputs, unsigned char input_number, state *outputs);

char digi_input(state *inputs, unsigned char input_number, state *outputs);

char but0(state *inputs, unsigned char input_number, state *outputs);

char but1(state *inputs, unsigned char input_number, state *outputs);

char motor(state *inputs, unsigned char input_number, state *outputs);

char D_flipflop(state *inputs, unsigned char input_number, state *outputs);

char repro(state *inputs, unsigned char input_number, state *outputs);

char servo(state *inputs, unsigned char input_number, state *outputs);

#endif